"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.schemaPlugin = void 0;
const tslib_1 = require("tslib");
const object_hash_1 = tslib_1.__importDefault(require("object-hash"));
const schema_1 = require("@nexus/schema");
const utils_1 = require("graphql-shield/dist/utils");
const graphql_shield_1 = require("graphql-shield");
function normalizeOptions(options) {
    if (typeof options.fallbackError === 'string') {
        options.fallbackError = new Error(options.fallbackError);
    }
    return {
        debug: options.debug !== undefined ? options.debug : false,
        allowExternalErrors: utils_1.withDefault(false)(options.allowExternalErrors),
        fallbackRule: utils_1.withDefault(graphql_shield_1.allow)(options.fallbackRule),
        fallbackError: utils_1.withDefault(new Error('Not Authorised!'))(options.fallbackError),
        hashFunction: utils_1.withDefault(object_hash_1.default)(options.hashFunction),
    };
}
function schemaPlugin(settings) {
    const options = normalizeOptions(settings.options || {});
    return schema_1.plugin({
        name: 'Nexus Schema Shield Plugin',
        onCreateFieldResolver(config) {
            const parentTypeName = config.parentTypeConfig.name;
            const fieldName = config.fieldConfig.name;
            let rule;
            const rules = settings.rules;
            if (utils_1.isRuleFunction(rules)) {
                rule = rules;
            }
            else {
                const typeRules = rules[parentTypeName];
                if (utils_1.isRuleFunction(typeRules)) {
                    rule = typeRules;
                }
                else if (utils_1.isRuleFieldMap(typeRules)) {
                    const defaultTypeRule = typeRules['*'];
                    rule = utils_1.withDefault(defaultTypeRule || options.fallbackRule)(typeRules[fieldName]);
                }
                else {
                    rule = options.fallbackRule;
                }
            }
            return async (root, args, ctx, info, next) => {
                // Cache
                if (!ctx) {
                    ctx = {};
                }
                if (!ctx._shield) {
                    ctx._shield = {
                        cache: {},
                    };
                }
                // Execution
                try {
                    const res = await rule.resolve(root, args, ctx, info, options);
                    if (res === true) {
                        return await next(root, args, ctx, info);
                    }
                    else if (res === false) {
                        if (typeof options.fallbackError === 'function') {
                            return await options.fallbackError(null, root, args, ctx, info);
                        }
                        return options.fallbackError;
                    }
                    else {
                        return res;
                    }
                }
                catch (err) {
                    if (options.debug) {
                        throw err;
                    }
                    else if (options.allowExternalErrors) {
                        return err;
                    }
                    else {
                        if (typeof options.fallbackError === 'function') {
                            return await options.fallbackError(err, root, args, ctx, info);
                        }
                        return options.fallbackError;
                    }
                }
            };
        },
    });
}
exports.schemaPlugin = schemaPlugin;
//# sourceMappingURL=schema.js.map